-- rslib4
-- Version 2015-04-27
-- these routines were written for Euphoria Version 3.1, but I'm using them now with Euphoria 4.0
-- from the Euphoria 4 POV, some things are outdated, others may need to be updated...
-- Euphoria: http://openeuphoria.org

-- I have written these routines for my own use, they are subject to change, their documentation is fragmentary, their functionality is not guaranteed.
-- This file is mainly included for the purpose of documentation. You are free to use this library, or create modified versions for your own purposes, but do so on your own peril.
-- Robert Schaechter, www.dunyazad-library.net


-- function begins(object d, object s)
-- function findeggs()
-- function findfiles(objecf f)
-- function findsfiles(object f)
-- function cut(object s, integer c)
-- function trim(object s)
-- function rtrim(object s)
-- function split(object s, object sep)
-- function splitb(object s)
-- function splitn(object s, object sep)
-- function split2(object s, object sep)
-- function splitx(object s, object sep)
-- function join(object s, atom sep)
-- function repl(object s, object sf, object sr)
-- function brack(object d, object s)
-- function readfile(object d)
-- function yesno()
-- procedure disp(object d)
-- procedure writefile(object s, object d)


-- note:
-- "strings" refers to sequences in which each element is an atom
-- "substrings" and "set of substrings" refers to sequences in which each element is a "string"


include get.e
include misc.e
include file.e
include sort.e
include wildcard.e


-- checks whether string d starts with string s (returns 0 or 1)
global function begins(object d, object s)
	integer x
	x = 0
	if length(d) >= length(s) then
		x = equal(d[1..length(s)], s)
	end if
	return x
end function


-- finds files whose names start with @
-- returns their names, and their names with @ stripped (names are sorted alphabetically)
global function findeggs()
	object x, y, d
	x = ""
	d = dir(current_dir())
	for j = 1 to length(d) do
		if begins(d[j][1], "@") then
			y = ""
			y = append(y, d[j][1])
			y = append(y, d[j][1][2..length(d[j][1])])
			x = append(x, y)
		end if
	end for
	x = sort(x)
	return x
end function


-- finds files whose names match the search string (may contain wildcard characters)
-- search string may be sequence of strings
-- returns their names, sorted alphabetically
global function findfiles(object f)
	object x, d
	x = {}
	if atom(f[1]) then f = {f} end if

	for i = 1 to length(f) do
		d = dir(f[i])
		if not atom(d) then
			for j = 1 to length(d) do
				if not equal (d[j][2], "d") then
					x = append(x, d[j][1])
				end if
			end for
		end if
	end for

	-- clumsy method to have files sorted by name and extension, not name.extension
	for i = 1 to length(x) do
		for j = length(x[i]) to 1 by -1 label "ex1" do
			if x[i][j] = '.' then
				x[i][j] = ''
				exit "ex1"
			end if
		end for
	end for

	x = sort(x)

	for i = 1 to length(x) do
		for j = length(x[i]) to 1 by -1 label "ex2" do
			if x[i][j] = '' then
				x[i][j] = '.'
				exit "ex2"
			end if
		end for
	end for

	return x
end function


-- used in findsfiles()
function tree()
	object subs
	object d
	integer ia, io

	subs = ""
	d = dir(current_dir())
	if atom(d) then return subs end if

	if equal(d[1][1], ".") then ia = 3 else ia = 1 end if
	for j = ia to length(d) do
		if equal(d[j][2], "d") then subs = append(subs, d[j][1]) end if
	end for

	ia = 1
	io = length(subs)
	if io = 0 then return subs end if

	while 1 do
		for i = ia to io do
			d = dir(subs[i])
			for j = 3 to length(d) do
				if equal(d[j][2], "d") then subs = append(subs, subs[i] & "\\" & d[j][1]) end if
			end for
		end for
		if length(subs) = io then exit end if

		ia = io + 1
		io = length(subs)
	end while

	subs = sort(subs)
	return subs
end function


-- as findfiles, but includes subdirectories
global function findsfiles(object f)
	object subs  -- list of subdirectories
	object x, xx
	if atom(f[1]) then f = {f} end if

	subs = tree()
	x = findfiles(f)

	for j = 1 to length(subs) do
		-- for Unix: slash, not backslash!
		for i = 1 to length(f) do
			xx = findfiles(subs[j] & "\\" & f[i])
			for jj = 1 to length(xx) do
				x = append(x, subs[j] & "\\" & xx[jj])
			end for
		end for
	end for

	-- clumsy method to have files sorted by name and extension, not name.extension
	for i = 1 to length(x) do
		for j = length(x[i]) to 1 by -1 label "ex1" do
			if x[i][j] = '.' then
				x[i][j] = ''
				exit "ex1"
			end if
		end for
	end for

	x = sort(x)

	for i = 1 to length(x) do
		for j = length(x[i]) to 1 by -1 label "ex2" do
			if x[i][j] = '' then
				x[i][j] = '.'
				exit "ex2"
			end if
		end for
	end for

	return x
end function


-- splits sequence s in two parts at position c: 1 to c / c+1 to end
-- with c negative: c characters from end
global function cut(object s, integer c)
	object x
	x = ""
	if c < 0 then
		c = length(s) + c
		if c < 0 then c = 0 end if
	end if
	if c > length(s) then c = length(s) end if
	x = append(x, s[1..c])
	x = append(x, s[c+1..length(s)])
	return x
end function


-- strips leading and trailing characters with ASCII codes <= 32 from sequence s,
-- and also from its elements, if these are sequences
global function trim(object s)
	integer first
	integer last
	first = 0
	last = 0
	for j = 1 to length(s) do
		if atom(s[j]) then
			if s[j] > 32 then
				last = j
				if first = 0 then first = j end if
			end if
		else
			last = j
			if first = 0 then first = j end if
			s[j] = trim(s[j])
		end if
	end for
	if first then
		s = s[first..last]
	else
		s = {}
	end if
	return s
end function


-- strips trailing characters with ASCII codes <= 32 from sequence s,
-- and also from its elements, if these are sequences
global function rtrim(object s)
	integer last
	last = 0
	for j = 1 to length(s) do
		if atom(s[j]) then
			if s[j] > 32 then last = j end if
		else
			last = j
			s[j] = rtrim(s[j])
		end if
	end for
	if last then
		s = s[1..last]
	else
		s = {}
	end if
	return s
end function


-- splits string s into substrings at the occurences of sep
-- if sep is an atom, it will not be included in the substrings
-- if sep is a sequence, it will be included
global function split(object s, object sep)
	object feld
	object x
	integer a
	integer i
	feld = ""
	x = ""
	if atom(sep) then
		for j = 1 to length(s) do
			if s[j] = sep then
				x = append(x, feld)
				feld = ""
			else
				feld = feld & s[j]
			end if
		end for
	else
		a = length(sep) - 1
		i = 1
		while i < length(s) - a do
			if equal(s[i..i+a], sep) then
				x = append(x, feld & sep)
				feld = ""
				i = i + a + 1
			else
				feld = feld & s[i]
				i += 1
			end if
		end while
	end if
	x = append(x, feld)
	return x
end function


-- splits string s into substrings, separated at space characters
-- several consecutive space characters are treated as one
global function splitb(object s)
	object feld
	object x
	integer a
	feld = ""
	x = ""
	a = 1

	for j = 1 to length(s) do
		if s[j] = ' ' then
			if a then
				x = append(x, feld)
				feld = ""
			end if
			a = 0
		else
			feld = feld & s[j]
			a = 1
		end if
	end for

	x = append(x, feld)
	return x
end function


-- splits string s into n substrings, separated at the occurence of sep, which must be a sequence
global function splitn(object s, object sep)
	object x
	integer a
	x = {}

	while length(s) do
		a = match(sep, s)
		if a then
			x = append(x, s[1..a-1])
			s = s[a+2..$]
		else
			x = append(x, s)
			exit
		end if
	end while
	return x
end function


-- splits string s into two substrings at the first occurence of sep (which must be an atom)
-- if sep is not found, the first returned substring is empty
global function split2(object s, atom sep)
	object x
	integer n
	x = {"",""}
	n = 0
	for j=1 to length(s) do
		if s[j] = sep then
			n = j
			exit
		end if
	end for

	if n = 0 then
		x[2] = s
	else

		x[1] = s[1..n-1]
		x[2] = s[n+1..$]
	end if
	return x
end function


-- splits string s into two substrings at the last occurence of sep (which must be an atom)
-- if sep is not found, the second returned substring is empty
global function splitx(object s, atom sep)
	object x
	integer n
	x = {"",""}
	n = 0
	for j= length(s) to 1 by -1 do
		if s[j] = sep then
			n = j
			exit
		end if
	end for

	if n = 0 then
		x[1] = s
	else

		x[1] = s[1..n-1]
		x[2] = s[n+1..$]
	end if
	return x
end function


-- joins substrings to one string
-- if sep = 0, no characters are inserted between substrings
-- if sep = ' ' then space characters are inserted if substring lines are not empty
-- with other values of sep, this atom is inserted between all substrings, empty or not
global function join(object s, atom sep)
	object x
	x = ""
	for j = 1 to length(s) do
		if sep and j > 1 then
			if sep = ' ' then
				if length(s[j-1]) then x = x & ' ' end if
			else
				x = x & sep
			end if
		end if
		x = x & s[j]
	end for
	return x
end function


-- replaces substring sf with substring sr within string s
global function repl(object s, object sf, object sr)
	integer i, l
	i = 1
	l = length(sf)

	while i do
		i = match_from(sf, s, i)
		if i then
			s = s[1..i-1] & sr & s[i+l..$]
		end if
	end while
	return s
end function


-- reduces text within brackets to single elements of a string
-- the result is not a string anymore, as its elements contain sequences!
-- s[1] is opening bracket, s[2] is closing bracket (must be atoms)
global function brack(object d, object s)
	sequence x, x1
	integer flag
	x = ""
	x1 = ""
	flag = 0
	for j = 1 to length(d) do
		if flag then
			x1 = append(x1, d[j])
			if d[j] = s[2] then
				flag = 0
				x = append(x, x1)
				x1 = ""
			end if
		else
			if d[j] = s[1] then
				flag = 1
				x1 = append(x1, d[j])
			else
				x = append(x, d[j])
			end if
		end if
	end for
	return x
end function


-- reads text file d line by line into a set of substrings
-- returns -1 if file doesn't exist
global function readfile(object d)
	integer Din
	object line
	object s
	s = {}
	Din = open(d, "r")
	if Din = -1 then
		s = -1
		return s
	else
		while 1 do
			line = gets(Din)
			if atom(line) then
				close(Din)
				s = rtrim(s)
				return s
			end if
			s = append(s, line)
		end while
	end if
end function


-- prompts user to press Y or N key (or ENTER or ESC key)
-- returns 1 for Y or ENTER, 0 for N or ESC
global function yesno()
	integer yes
	integer ky
	yes = -1
	puts(1, "(Y/N) ")
	while yes = -1 do
		ky = wait_key()
		if ky = 27 or ky = 78 or ky = 110 then
			yes = 0
		elsif ky = 13 or ky = 89 or ky = 121 then
			yes = 1
		end if
	end while
	puts(1, "\n")
	return yes
end function


-- simple shortcut for puts command to write a line of text to the screen (stdout)
global procedure disp(object d)
	puts(1, d & "\n")
end procedure


-- writes set of substrings to file d
global procedure writefile(object s, object d)
	integer Dout
	Dout = open(d, "w")
	if Dout = -1 then
		puts(1, "Error: could not write file " & d & "\n(Invalid file name, or write protected)\n")
		abort(1)
	else
		for j = 1 to length(s) do
			puts(Dout, s[j] & "\n")
		end for
		close(Dout)
	end if
end procedure